@@ -42,6 +42,14 @@
 #include "attributes.h"
 #include <string.h>
 
+#ifndef VOMS_MAYBECONST
+#if defined(D2I_OF)
+#define VOMS_MAYBECONST const
+#else
+#define VOMS_MAYBECONST
+#endif
+#endif
+
 int i2d_AC_SEQ(AC_SEQ *a, unsigned char **pp)
 {
   M_ASN1_I2D_vars(a);
@@ -59,7 +67,7 @@ int i2d_AC_SEQ(AC_SEQ *a, unsigned char **pp)
   M_ASN1_I2D_finish();
 }
 
-AC_SEQ *d2i_AC_SEQ(AC_SEQ **a, unsigned char **pp, long length)
+AC_SEQ *d2i_AC_SEQ(AC_SEQ **a, VOMS_MAYBECONST unsigned char **pp, long length)
 {
   M_ASN1_D2I_vars(a, AC_SEQ *, AC_SEQ_new);
 
@@ -96,7 +104,7 @@ int i2d_AC_TARGETS(AC_TARGETS *a, unsigned char **pp)
   M_ASN1_I2D_put_SEQUENCE(a->targets, i2d_AC_TARGET);
   M_ASN1_I2D_finish();
 }
-AC_TARGETS *d2i_AC_TARGETS(AC_TARGETS **a, unsigned char **pp, long length)
+AC_TARGETS *d2i_AC_TARGETS(AC_TARGETS **a, VOMS_MAYBECONST unsigned char **pp, long length)
 {
   M_ASN1_D2I_vars(a, AC_TARGETS *, AC_TARGETS_new);
 
@@ -140,7 +148,7 @@ int i2d_AC_TARGET(AC_TARGET *a, unsigned char **pp)
   M_ASN1_I2D_finish();
 }
 
-AC_TARGET *d2i_AC_TARGET(AC_TARGET **a, unsigned char **pp, long length)
+AC_TARGET *d2i_AC_TARGET(AC_TARGET **a, VOMS_MAYBECONST unsigned char **pp, long length)
 {
   M_ASN1_D2I_vars(a, AC_TARGET *, AC_TARGET_new);
 
@@ -175,8 +183,6 @@ void AC_TARGET_free(AC_TARGET *a)
 
 int i2d_AC_CERTS(AC_CERTS *a, unsigned char **pp)
 {
-  //  int v1=0, v2=0, v3=0;
-
   M_ASN1_I2D_vars(a);
   M_ASN1_I2D_len_SEQUENCE(a->stackcert, i2d_X509);
   M_ASN1_I2D_seq_total();
@@ -184,7 +190,7 @@ int i2d_AC_CERTS(AC_CERTS *a, unsigned char **pp)
   M_ASN1_I2D_finish();
 }
 
-AC_CERTS *d2i_AC_CERTS(AC_CERTS **a, unsigned char **pp, long length)
+AC_CERTS *d2i_AC_CERTS(AC_CERTS **a, VOMS_MAYBECONST unsigned char **pp, long length)
 {
   M_ASN1_D2I_vars(a, AC_CERTS *, AC_CERTS_new);
 
@@ -229,7 +235,7 @@ int i2d_AC_ATTRIBUTE(AC_ATTRIBUTE *a, unsigned char **pp)
   M_ASN1_I2D_finish();
 }
 
-AC_ATTRIBUTE *d2i_AC_ATTRIBUTE(AC_ATTRIBUTE **a, unsigned char **pp, long length)
+AC_ATTRIBUTE *d2i_AC_ATTRIBUTE(AC_ATTRIBUTE **a, VOMS_MAYBECONST unsigned char **pp, long length)
 {
   M_ASN1_D2I_vars(a, AC_ATTRIBUTE *, AC_ATTRIBUTE_new);
 
@@ -279,7 +285,7 @@ int i2d_AC_ATT_HOLDER(AC_ATT_HOLDER *a, unsigned char **pp)
 }
 
 
-AC_ATT_HOLDER *d2i_AC_ATT_HOLDER(AC_ATT_HOLDER **a, unsigned char **pp, long length)
+AC_ATT_HOLDER *d2i_AC_ATT_HOLDER(AC_ATT_HOLDER **a, VOMS_MAYBECONST unsigned char **pp, long length)
 {
   M_ASN1_D2I_vars(a, AC_ATT_HOLDER *, AC_ATT_HOLDER_new);
 
@@ -321,7 +327,7 @@ int i2d_AC_FULL_ATTRIBUTES(AC_FULL_ATTRIBUTES *a, unsigned char **pp)
   M_ASN1_I2D_finish();
 }
 
-AC_FULL_ATTRIBUTES *d2i_AC_FULL_ATTRIBUTES(AC_FULL_ATTRIBUTES **a, unsigned char **pp, long length)
+AC_FULL_ATTRIBUTES *d2i_AC_FULL_ATTRIBUTES(AC_FULL_ATTRIBUTES **a, VOMS_MAYBECONST unsigned char **pp, long length)
 {
   M_ASN1_D2I_vars(a, AC_FULL_ATTRIBUTES *, AC_FULL_ATTRIBUTES_new);
 
@@ -350,14 +356,6 @@ void AC_FULL_ATTRIBUTES_free(AC_FULL_ATTRIBUTES *a)
   OPENSSL_free(a);
 }
 
-#if 0
-IMPL_STACK(AC_ATTRIBUTE)
-IMPL_STACK(AC_ATT_HOLDER)
-/*
-IMPL_STACK(AC_FULL_ATTRIBUTES)
-*/
-#endif
-
 static char *norep()
 {
   static char *buffer = 0;
@@ -412,11 +410,13 @@ void *targets_s2i(UNUSED(struct v3_ext_method *method), UNUSED(struct v3_ext_ctx
 {
   char *pos;
   char *list = strdup(data);
+  char *back = list;
   AC_TARGETS *a = AC_TARGETS_new();
 
   int attlist;
   do {
     pos = strchr(list, ',');
+
     if (pos)
       *pos = '\0';
     {
@@ -441,9 +441,11 @@ void *targets_s2i(UNUSED(struct v3_ext_method *method), UNUSED(struct v3_ext_ctx
       list = ++pos;
   } while (pos);
 
+  free(back);
   return a;
 
  err:
+  free(back);
   AC_TARGETS_free(a);
   return NULL;    
 
