@@ -52,8 +52,11 @@ extern "C" {
 #include <openssl/objects.h>
 #include <openssl/asn1.h>
 #include <openssl/evp.h>
+#include <openssl/pkcs12.h>
+
 #include "credentials.h"
 #include "sslutils.h"
+#include "gssapi_compat.h"
 
 #ifndef NOGLOBUS
 #ifdef HAVE_GLOBUS_MODULE_ACTIVATE
@@ -82,29 +85,14 @@ extern int InitProxyCertInfoExtension(int);
 extern bool retrieve(X509 *cert, STACK_OF(X509) *chain, recurse_type how, 
 		     std::string &buffer, std::string &vo, std::string &file, 
 		     std::string &subject, std::string &ca, verror_type &error);
-extern "C" {
-extern char *Decode(const char *, int, int *);
-extern char *Encode(const char *, int, int *, int);
-}
+
+static std::string parse_commands(const std::string& commands);
 
 extern int AC_Init(void);
 
 std::map<vomsdata*, vomsspace::internal*> privatedata;
 pthread_mutex_t privatelock = PTHREAD_MUTEX_INITIALIZER;
 
-vomsdata::Initializer::Initializer(Initializer &) {}
-vomsdata::Initializer::Initializer()
-{
-#ifdef NOGLOBUS
-  //  openssl_initialize();
-  SSLeay_add_all_algorithms();
-  ERR_load_crypto_strings();
-
-  (void)AC_Init();
-  InitProxyCertInfoExtension(1);
-#endif
-}
-
 static bool initialized = false;
 
 void vomsdata::seterror(verror_type err, std::string message)
@@ -127,16 +115,16 @@ vomsdata::vomsdata(std::string voms_dir, std::string cert_dir) :  ca_cert_dir(ce
                                                                   extra_data(""),
                                                                   ver_type(VERIFY_FULL),
                                                                   retry_count(1),
-                                                                  verificationtime(0)
+                                                                  verificationtime(0),
+                                                                  vdp(NULL)
 {
 #ifndef NOGLOBUS
    (void)globus_thread_once(&l_globus_once_control, l_init_globus_once_func);
 #endif
 
    if (!initialized) {
-     initialized = 1;
+     initialized = true;
 #ifdef NOGLOBUS
-     //  openssl_initialize();
      SSL_library_init();
      SSLeay_add_all_algorithms();
      ERR_load_crypto_strings();
@@ -145,6 +133,7 @@ vomsdata::vomsdata(std::string voms_dir, std::string cert_dir) :  ca_cert_dir(ce
      (void)AC_Init();
      InitProxyCertInfoExtension(1);
 #endif
+     PKCS12_PBE_add();
    }
 
   if (voms_cert_dir.empty()) {
@@ -178,8 +167,6 @@ vomsdata::vomsdata(std::string voms_dir, std::string cert_dir) :  ca_cert_dir(ce
   if (vdir)
     (void)closedir(vdir);
 
-  duration = 0;
-
   vomsspace::internal *data = new vomsspace::internal();
   pthread_mutex_lock(&privatelock);
   privatedata[this] = data;
@@ -194,12 +181,13 @@ vomsdata::~vomsdata()
   (void)privatedata.erase(this);
   pthread_mutex_unlock(&privatelock);
   delete data;
+
 }
 
 std::string vomsdata::ServerErrors(void)
 {
   std::string err = serverrors;
-  serverrors="";
+  serverrors.clear();
 
   return err;
 }
@@ -231,7 +219,7 @@ void vomsdata::SetVerificationType(verify_type t)
 
 void vomsdata::ResetOrder(void)
 {
-  ordering="";
+  ordering.clear();
 }
 
 void vomsdata::Order(std::string att)
@@ -243,36 +231,19 @@ bool vomsdata::ContactRaw(std::string hostname, int port, std::string servsubjec
   return ContactRaw(hostname, port, servsubject, command, raw, version, -1);
 }
 
-bool vomsdata::ContactRaw(std::string hostname, int port, std::string servsubject, std::string command, std::string &raw, int& version, int timeout)
+bool vomsdata::InterpretOutput(const std::string &message, std::string& output)
 {
-  std::string buffer;
-  std::string subject, ca;
-  std::string lifetime;
-
-  std::string comm;
-  std::string targs;
   answer a;
-
-  for (std::vector<std::string>::iterator i = targets.begin(); 
-       i != targets.end(); i++) {
-    if (i == targets.begin())
-      targs = *i;
-    else
-      targs += std::string(",") + *i;
-  }
-
-  comm = XML_Req_Encode(command, ordering, targs, duration);
-
-  if (!contact(hostname, port, servsubject, comm, buffer, subject, ca, timeout))
-    return false;
   
-  if (XML_Ans_Decode(buffer, a)) {
+  if (XML_Ans_Decode(message, a)) {
     bool result = true;
+
     if (!a.ac.empty()) {
-      buffer = a.ac;
+      output = a.ac;
       if (a.errs.size() != 0) {
-        for (std::vector<errorp>::iterator i = a.errs.begin();
-             i != a.errs.end(); i++) {
+        std::vector<errorp>::const_iterator end = a.errs.end();
+        for (std::vector<errorp>::const_iterator i = a.errs.begin();
+             i != end; ++i) {
           serverrors += i->message;
           if (i->num > ERROR_OFFSET)
             result = false;
@@ -282,23 +253,56 @@ bool vomsdata::ContactRaw(std::string hostname, int port, std::string servsubjec
       }
     }
     else if (!a.data.empty()) {
-      buffer = a.data;
+      output = a.data;
     }
     if (!result && ver_type) {
       seterror(VERR_SERVERCODE, "The server returned an error.");
       return false;
     }
-    raw = buffer;
   }
   else {
     seterror(VERR_FORMAT, "Server Answer was incorrectly formatted.");
     return false;
   }
 
-  version = 1;
   return true;
 }
 
+bool vomsdata::ContactRaw(std::string hostname, int port, std::string servsubject, std::string command, std::string &raw, int& version, int timeout)
+{
+  std::string buffer;
+  std::string subject, ca;
+  std::string lifetime;
+
+  std::string comm;
+  std::string targs;
+
+  version = 1;
+
+  /* Try REST connection first */
+  bool ret = ContactRESTRaw(hostname, port, command, raw, version, timeout);
+
+  if (ret)
+    return ret;
+
+  std::vector<std::string>::const_iterator end = targets.end();
+  std::vector<std::string>::const_iterator begin = targets.begin();
+  for (std::vector<std::string>::const_iterator i = begin; i != end; ++i) {
+    if (i == begin)
+      targs = *i;
+    else
+      targs += std::string(",") + *i;
+  }
+
+  comm = XML_Req_Encode(command, ordering, targs, duration);
+
+  if (!contact(hostname, port, servsubject, comm, buffer, subject, ca, timeout))
+    return false;
+
+  version = 1;
+  return InterpretOutput(buffer, raw);
+}
+
 static X509 *get_own_cert()
 {
   char *certname = NULL;
@@ -314,11 +318,64 @@ static X509 *get_own_cert()
 }
 
 
+bool vomsdata::ContactRESTRaw(const std::string& hostname, int port, const std::string& command, std::string& raw, UNUSED(int version), int timeout)
+{
+  std::string temp;
+
+  std::string realCommand = "GET /generate-ac?fqans="+ parse_commands(command);
+
+  realCommand += "&lifetime="+ stringify(duration, temp);
+
+  if (!ordering.empty())
+    realCommand +="&order=" + ordering;
+
+  if (targets.size() != 0) {
+    std::string targs;
+
+    std::vector<std::string>::const_iterator end = targets.end();
+    std::vector<std::string>::const_iterator begin = targets.begin();
+
+    for (std::vector<std::string>::const_iterator i = targets.begin(); i != end; ++i) {
+      if (i == begin)
+        targs = *i;
+      else
+        targs += std::string(",") + *i;
+    }
+
+    realCommand +="&targets="+targs;
+  }
+
+  realCommand += std::string(" HTTP/1.0\n") + 
+    "User-Agent: voms APIs 2.0\nAccept: */*\nHost: "+
+    hostname+":"+ stringify(port,temp) +"\n\n";
+
+  std::string user, userca, output;
+  bool res = contact(hostname, port, "", realCommand, output, user, userca, timeout);
+
+  bool ret = false;
+
+  if (res) {
+    std::string::size_type pos = output.find("<?xml");
+
+    if (pos != std::string::npos)
+      ret = InterpretOutput(output.substr(pos), raw);
+
+    if (ret) 
+      if (!(output.substr(0,12) == "HTTP/1.1 200"))
+        return false;
+    
+    return ret;
+  }
+
+  return ret;
+}
+
 bool vomsdata::Contact(std::string hostname, int port, std::string servsubject, std::string command) {
   return Contact(hostname, port, servsubject, command, -1);
 }
 
-bool vomsdata::Contact(std::string hostname, int port, std::string servsubject, std::string command, int timeout)
+
+bool vomsdata::Contact(std::string hostname, int port, UNUSED(std::string servsubject), std::string command, int timeout)
 {
   std::string subject, ca;
   char *s = NULL, *c = NULL;
@@ -341,7 +398,6 @@ bool vomsdata::Contact(std::string hostname, int port, std::string servsubject,
         if (c && s) {
           ca = std::string(c);
           subject = std::string(s);
-    
           voms v;
           
           result = verifydata(message, subject, ca, holder, v);
@@ -395,20 +451,19 @@ bool vomsdata::RetrieveFromCred(gss_cred_id_t cred, recurse_type how)
   X509 *cert;
   STACK_OF(X509) *chain;
 
-  cert = decouple_cred(cred, &chain);
+  chain = ((gss2_cred_id_desc *)cred)->cred_handle->cert_chain;
+  cert = ((gss2_cred_id_desc *)cred)->cred_handle->cert;
 
   return Retrieve(cert, chain, how);
 }
 
-bool vomsdata::RetrieveFromCtx(gss_ctx_id_t cred, recurse_type how)
+bool vomsdata::RetrieveFromCtx(UNUSED(gss_ctx_id_t cred), UNUSED(recurse_type how))
 {
   return false;
 }
 
 bool vomsdata::RetrieveFromProxy(recurse_type how)
 {
-  gss_cred_id_t cred = 0L;
-
   char *outfile = NULL;
 
   if (determine_filenames(NULL, NULL, &outfile, NULL, NULL, 0)) {
@@ -435,6 +490,22 @@ bool vomsdata::Retrieve(X509_EXTENSION *ext)
   return ret;
 }
 
+bool vomsdata::Retrieve(AC *ac)
+{
+  verify_type v = ver_type;
+
+  ver_type = (verify_type)((int) ver_type & (~VERIFY_ID));
+
+  voms vv;
+
+  bool ret = verifydata(ac, "", "", NULL, vv);
+
+  if (ret)
+    data.push_back(vv);
+
+  return ret;
+}
+
 bool vomsdata::Retrieve(X509 *cert, STACK_OF(X509) *chain, recurse_type how)
 {
   bool ok = false;
@@ -470,15 +541,9 @@ bool vomsdata::Import(std::string buffer)
   unsigned char *buftmp, *copy;
 #endif
 
-  char *str;
-  int len;
+  buffer = Decode(buffer);
 
-  str = Decode(buffer.c_str(), buffer.size(), &len);
-  if (str) {
-    buffer = std::string(str, len);
-    free(str);
-  }
-  else {
+  if (buffer.empty()) {
     seterror(VERR_FORMAT, "Malformed input data.");
     return false;
   }
@@ -521,11 +586,14 @@ bool vomsdata::Export(std::string &buffer)
   std::string temp;
 
   if (data.empty()) {
-    buffer= "";
+    buffer.clear();
     return true;
   }
 
-  for (std::vector<voms>::iterator v=data.begin(); v != data.end(); v++) {
+
+  std::vector<voms>::const_iterator end = data.end();
+
+  for (std::vector<voms>::const_iterator v=data.begin(); v != end; ++v) {
     /* Dump owner's certificate */
     int l;
     unsigned char *xtmp, *xtmp2;
@@ -560,14 +628,10 @@ bool vomsdata::Export(std::string &buffer)
     }
   }
 
-  char *str;
-  int len;
-  str = Encode(result.c_str(), result.size(), &len, 0);
-  if (str) {
-    buffer = std::string(str, len);
-    free(str);
+  buffer = Encode(result, 0);
+
+  if (!buffer.empty())
     return true;
-  }
   else
     return false;
 }
@@ -583,7 +647,7 @@ bool vomsdata::DefaultData(voms &d)
   return true;
 }
 
-bool vomsdata::loadfile(std::string filename, uid_t uid, gid_t gid)
+bool vomsdata::loadfile(std::string filename, UNUSED(uid_t uid), UNUSED(gid_t gid))
 {
   struct stat stats;
 
@@ -601,15 +665,6 @@ bool vomsdata::loadfile(std::string filename, uid_t uid, gid_t gid)
     return false;
   }
 
-#if 0
-  //  This is no longer an issue
-  if (stats.st_mode & (S_IWGRP | S_IWOTH)) {
-    seterror(VERR_DIR, std::string("Wrong permissions on file: ") + filename + 
-             "\nWriting permissions are allowed only for the owner\n");
-    return false;
-  }
-#endif
-
   if (stats.st_mode & S_IFREG)
     return loadfile0(filename, 0, 0);
   else {
@@ -660,7 +715,7 @@ static bool empty(std::string c)
   return true;
 }
 
-bool vomsdata::loadfile0(std::string filename, uid_t uid, gid_t gid)
+bool vomsdata::loadfile0(std::string filename, UNUSED(uid_t uid), UNUSED(gid_t gid))
 {
   struct contactdata data;
 
@@ -718,7 +773,7 @@ bool vomsdata::loadfile0(std::string filename, uid_t uid, gid_t gid)
 bool vomsdata::LoadSystemContacts(std::string dir)
 {
   if (dir.empty())
-    dir = "/opt/glite/etc/vomses";
+    dir = "/etc/vomses";
 
   return loadfile(dir, 0, 0);
 }
@@ -755,13 +810,13 @@ bool vomsdata::LoadUserContacts(std::string dir)
 std::vector<contactdata>
 vomsdata::FindByAlias(std::string nick)
 {
-  std::vector<contactdata>::iterator beg = servers.begin(), end = servers.end();
+  std::vector<contactdata>::const_iterator beg = servers.begin(), end = servers.end();
   std::vector<contactdata> results;
 
   while (beg != end) {
     if (beg->nick == nick)
       results.push_back(*beg);
-    beg++;
+    ++beg;
   }
 
   return std::vector<contactdata>(results);
@@ -769,13 +824,13 @@ vomsdata::FindByAlias(std::string nick)
 
 std::vector<contactdata> vomsdata::FindByVO(std::string vo)
 {
-  std::vector<contactdata>::iterator beg = servers.begin(), end = servers.end();
+  std::vector<contactdata>::const_iterator beg = servers.begin(), end = servers.end();
   std::vector<contactdata> results;
 
   while (beg != end) {
     if (beg->vo == vo)
       results.push_back(*beg);
-    beg++;
+    ++beg;
   }
 
   return std::vector<contactdata>(results);
@@ -805,10 +860,10 @@ voms::voms(const voms &orig)
   
   ((struct realdata *)realdata)->attributes = 
     new std::vector<attributelist>(*(((struct realdata *)orig.realdata)->attributes));
+  vp = NULL;
 }
 
-
-voms::voms(): version(0), siglen(0), holder(NULL)
+voms::voms(): version(0), siglen(0), holder(NULL), vp(NULL)
 {
   realdata = (void *)calloc(1, sizeof(struct realdata));
 }
@@ -834,8 +889,9 @@ voms &voms::operator=(const voms &orig)
   custom    = orig.custom;
   fqan      = orig.fqan;
   serial    = orig.serial;
-  if (((struct realdata *)realdata)->ac)
-    AC_free(((struct realdata *)realdata)->ac);
+  vp        = NULL;
+  AC_free(((struct realdata *)realdata)->ac);
+
   ((struct realdata *)realdata)->ac = AC_dup(((struct realdata *)orig.realdata)->ac);
   holder = X509_dup(orig.holder);
   delete ((struct realdata *)realdata)->attributes;
@@ -876,12 +932,15 @@ vomsdata::vomsdata(const vomsdata &orig) : ca_cert_dir(orig.ca_cert_dir),
                                            serverrors(orig.serverrors),
                                            errmessage(orig.errmessage),
                                            retry_count(orig.retry_count),
-                                           verificationtime(orig.verificationtime)
+                                           verificationtime(orig.verificationtime),
+                                           vdp(NULL)
 {}
 
-int getMajorVersionNumber(void) {return 1;}
-int getMinorVersionNumber(void) {return 8;}
-int getPatchVersionNumber(void) {return 0;}
+extern "C" {
+int getVOMSMajorVersionNumber(void) {return 2;}
+int getVOMSMinorVersionNumber(void) {return 0;}
+int getVOMSPatchVersionNumber(void) {return 0;}
+}
 
 void vomsdata::SetRetryCount(int retryCount)
 {
@@ -901,11 +960,11 @@ std::vector<std::string> voms::GetTargets()
 
   STACK_OF(X509_EXTENSION) *exts = ac->acinfo->exts;
 
-  int nid3 = OBJ_txt2nid("idceTargets");
-  int pos4 = X509v3_get_ext_by_NID(exts, nid3, -1);
+  int nid = OBJ_txt2nid("idceTargets");
+  int pos = X509v3_get_ext_by_NID(exts, nid, -1);
 
-  if (pos4 >= 0) {
-    X509_EXTENSION *ex = sk_X509_EXTENSION_value(exts, pos4);
+  if (pos >= 0) {
+    X509_EXTENSION *ex = sk_X509_EXTENSION_value(exts, pos);
     AC_TARGETS *target = (AC_TARGETS *)X509V3_EXT_d2i(ex);
 
     if (target != NULL) {
@@ -924,13 +983,8 @@ std::vector<std::string> voms::GetTargets()
   return targets;
 }
 
-// Functionality removed due to ABI incompatibility
 bool vomsdata::LoadCredentials(X509 *cert, EVP_PKEY *pkey, STACK_OF(X509) *chain)
 {
-  X509           *ucert = NULL;
-  STACK_OF(X509) *cert_chain = NULL;
-  EVP_PKEY       *upkey = NULL;
-
   pthread_mutex_lock(&privatelock);
   vomsspace::internal *data = privatedata[this];
   pthread_mutex_unlock(&privatelock);
@@ -953,6 +1007,7 @@ bool vomsdata::LoadCredentials(X509 *cert, EVP_PKEY *pkey, STACK_OF(X509) *chain
      stack itself. */
   /* So, do the duplication by hand. */
   if (chain) {
+    sk_X509_pop_free(data->chain, X509_free);
     data->chain = sk_X509_new_null();
 
     if (data->chain) {
@@ -984,3 +1039,31 @@ bool vomsdata::LoadCredentials(X509 *cert, EVP_PKEY *pkey, STACK_OF(X509) *chain
   return true;
 }
 
+
+static void change(std::string &name, const std::string& from, const std::string& to) 
+{
+  std::string::size_type pos = name.find(from);
+
+  while (pos != std::string::npos) {
+    name = name.substr(0, pos) + to + name.substr(pos+from.length());
+    pos = name.find(from, pos+1);
+  }
+}
+
+static std::string parse_commands(const std::string& commands)
+{
+  if (commands[0] == '/')
+    return commands;
+
+  if (commands[0] == 'A')
+    return std::string("all");
+
+  std::string temp = commands;
+
+  change(temp, ":", "/Role=");
+  change(temp, "G/", "/");
+  change(temp, "B/", "/");
+  change(temp, "R/", "/Role=");
+
+  return temp;
+}
