@@ -30,16 +30,12 @@
 #include <string>
 #include <vector>
 
-
+#define NOGLOBUS
 extern "C" {
-#ifdef NOGLOBUS
 #ifndef GSSAPI_H_
 typedef void * gss_cred_id_t;
 typedef void * gss_ctx_id_t;
 #endif
-#else
-#include "gssapi.h"
-#endif
 
 #include <openssl/x509.h>
 #include <openssl/bio.h>
@@ -85,11 +81,13 @@ struct contactdata {   /*!< You must never allocate directly this structure.
   std::string  host;    /*!< The hostname of the server */
   std::string  contact; /*!< The subject of the server's certificate */
   std::string  vo;      /*!< The VO served by this server */
-  int          port;	       /*!< The port on which the server is listening */
+  int          port;	  /*!< The port on which the server is listening */
 
   int          version; /*!< The version of globus under which the server is running */ 
 };
 
+class vomspriv;
+
 struct voms {
   friend class vomsdata;
   int version;             /*!< 0 means data didn't originate from an AC */
@@ -130,6 +128,9 @@ public:
 public:
   std::vector<attributelist>& GetAttributes();   /*!< Generic attributes */
   std::vector<std::string> GetTargets();
+
+private:
+  vomspriv *vp;
 };
 
 enum recurse_type { 
@@ -181,6 +182,8 @@ enum verror_type {
 
 typedef bool (*check_sig)(X509 *, void *, verror_type &); /*!<*/
 
+class vomsdatapriv;
+
 struct vomsdata {
   private:
   class Initializer {
@@ -373,7 +376,6 @@ private:
   bool loadfile0(std::string, uid_t uid, gid_t gid);
   bool verifydata(std::string &message, std::string subject, std::string ca, 
                   X509 *holder, voms &v);
-  X509 *check(check_sig f, void *data); /*!< Unused.  Only left here for binary compatibility. */
   bool check_cert(X509 *cert);
   bool retrieve(X509 *cert, STACK_OF(X509) *chain, recurse_type how,
                 AC_SEQ **listnew, std::string &subject, std::string &ca,
@@ -385,18 +387,8 @@ private:
   
   void seterror(verror_type, std::string);
 
-  bool verifyac(X509 *, X509 *, AC*, voms&);
   bool check_sig_ac(X509 *, void *);
   X509 *check(void *);
-  bool my_conn(const std::string&, int, const std::string&, int,
-               const std::string&, std::string&, std::string&,
-               std::string&);
-  bool contact(const std::string&, int, const std::string&,
-               const std::string&, std::string&, std::string&,
-               std::string&);
-  bool my_conn(const std::string&, int, const std::string&, int,
-               const std::string&, std::string&, std::string&,
-               std::string&, int timeout);
   bool contact(const std::string&, int, const std::string&,
                const std::string&, std::string&, std::string&,
                std::string&, int timeout);
@@ -419,29 +411,6 @@ public:
              \param credential The credential from which to retrieve the certificate.
              \param how Recursion type
              \return failure (F) or success (T)*/
-#else
-  bool RetrieveFromCtx(void *context, recurse_type how); /*!< Gets VOMS information from the given globus context
-             \param context The context from which to retrieve the certificate.
-             \param how Recursion type
-             \return failure (F) or success (T)*/
-
-  bool RetrieveFromCred(void *credential, recurse_type how);  /*!< Gets VOMS information from the given globus credential
-             \param credential The credential from which to retrieve the certificate.
-             \param how Recursion type
-             \return failure (F) or success (T)*/
-  bool RetrieveFromCtx(gss_ctx_id_t context, recurse_type how) {
-    return RetrieveFromCtx((void*)context,how);
-  }/*!< Gets VOMS information from the given globus context
-             \param context The context from which to retrieve the certificate.
-             \param how Recursion type
-             \return failure (F) or success (T)*/
-
-  bool RetrieveFromCred(gss_cred_id_t credential, recurse_type how) {
-    return RetrieveFromCred((void*)credential, how);
-  } /*!< Gets VOMS information from the given globus credential
-             \param credential The credential from which to retrieve the certificate.
-             \param how Recursion type
-             \return failure (F) or success (T)*/
 #endif
 
   bool Retrieve(X509_EXTENSION *ext); /*!< Gets VOMS information from the given extension
@@ -453,11 +422,16 @@ public:
              \return failure (F) or success (T)*/
 
   bool Retrieve(FILE *file, recurse_type how); /*!< Gets VOMS information from a proxy saved as a file.
-                               \param the file
+                               \param file the file name
                                \param how Recursion type
                                \return failure (F) or success (T)
 
                                Note: Does NOT verify that the proxy is valid.  Such verification must be obtained through other means. */
+
+  bool Retrieve(AC *ac);/*!< Gets VOMS information from the AC
+             \param ext The extension to parse.
+             \return failure (F) or success (T) */
+
   ~vomsdata();
 private:
   //  X509 *check_file(void *);
@@ -474,9 +448,6 @@ private:
 public:
   void SetRetryCount(int retryCount);
   
-private:
-  STACK_OF(X509) *load_chain(BIO *in);
-                                
 public:
   void SetVerificationTime(time_t);
                                   
@@ -486,11 +457,20 @@ private:
 
 public:
   bool LoadCredentials(X509*, EVP_PKEY *, STACK_OF(X509) *);
+  bool ContactRESTRaw(const std::string&, int, const std::string&, std::string&, int, int);
+
+private:
+  bool InterpretOutput(const std::string&, std::string&);
+
+private:
+  vomsdatapriv *vdp;
 };
 
 
-int getMajorVersionNumber(void);
-int getMinorVersionNumber(void);
-int getPatchVersionNumber(void);
+extern "C" {
+int getVOMSMajorVersionNumber(void);
+int getVOMSMinorVersionNumber(void);
+int getVOMSPatchVersionNumber(void);
+}
 
 #endif
