@@ -26,14 +26,15 @@
 #ifndef VOMS_APIC_H
 #define VOMS_APIC_H
 
-#ifdef NOGLOBUS
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+#define NOGLOBUS
 #ifndef GSSAPI_H_
 typedef void * gss_cred_id_t;
 typedef void * gss_ctx_id_t;
 #endif
-#else
-#include <gssapi.h>
-#endif
 
 #include <openssl/x509.h>
 #include <time.h>
@@ -398,6 +399,12 @@ extern int VOMS_RetrieveFromProxy(int how, struct vomsdata *vd, int *error); /*!
              \param error RETURN PARAMETER: Qualifies the error message
              \return failure (0) or success (<>0)*/
 
+extern int VOMS_RetrieveFromAC(AC*ac, struct vomsdata *vd, int *error);  /*!< Gets VOMS information from an existing globus proxy
+             \param ac AC from which to get the credentials
+             \param vd RETURN PARAMETER: contains the data returned by the connection
+             \param error RETURN PARAMETER: Qualifies the error message
+             \return failure (0) or success (<>0)*/
+
 extern int VOMS_GetAttributeSourcesNumber(struct voms *v, struct vomsdata *vd, int *error);
 extern int VOMS_GetAttributeSourceHandle(struct voms *v, int num, struct vomsdata *vd, int *error);
 extern const char *VOMS_GetAttributeGrantor(struct voms *v, int handle, struct vomsdata *vd, int *error);
@@ -416,6 +423,12 @@ extern char **VOMS_GetTargetsList(struct voms *v, struct vomsdata *vd, int *erro
 extern void VOMS_FreeTargetsList(char **);
 
 extern int VOMS_SetTimeout(int t, struct vomsdata *vd, int *error);
+extern int VOMS_LoadCredentials(X509 *cert, EVP_PKEY *pkey, STACK_OF(X509)* chain, struct vomsdata *vd, int *error);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif
 
 
