@@ -65,7 +65,8 @@ public class PKIStore implements VOMSTrustStore {
     private Hashtable signings     = null;
     private Hashtable lscfiles     = null;
     private Hashtable vomscerts    = null;
-    private long      lastmodified = 0;
+    private Hashtable namespaces   = null;
+
     private int       instances    = 1;
 
     private static Logger logger = Logger.getLogger(PKIStore.class.getName());
@@ -80,10 +81,11 @@ public class PKIStore implements VOMSTrustStore {
      */
     public static final int TYPE_CADIR = 2;
 
-    private static final int CERT = 1;
-    private static final int CRL  = 2;
-    private static final int SIGN = 3;
-    private static final int LSC  = 4;
+    private static final int CERT      = 1;
+    private static final int CRL       = 2;
+    private static final int SIGN      = 3;
+    private static final int LSC       = 4;
+    private static final int NAMESPACE = 5;
     private static final int HASHCAPACITY = 75;
 
     private boolean aggressive = false;
@@ -146,6 +148,10 @@ public class PKIStore implements VOMSTrustStore {
     }
 
 
+    public Hashtable getNamespaces() {
+        return namespaces;
+    }
+
     static {
         if (Security.getProvider("BC") == null) {
             Security.addProvider(new BouncyCastleProvider());
@@ -166,13 +172,13 @@ public class PKIStore implements VOMSTrustStore {
     public synchronized void refresh() {
         PKIStore newReader = null;
 
-        File f = new File(certDir);
-
         /* The code below did not work.  In place changes to a file do not 
            change the lastmodified date of the directory, only the 
            lastaccessed, and java does not provide a way to determine the
            latter. */
         /*
+        File f = new File(certDir);
+
         if (f.lastModified() == lastmodified) {
             logger.debug("No changes to directory -- Do not refresh");
             return;
@@ -210,7 +216,12 @@ public class PKIStore implements VOMSTrustStore {
             vomscerts.clear();
             vomscerts = newReader.vomscerts;
             newReader.vomscerts = null;
-            lastmodified = f.lastModified();
+
+            namespaces.clear();
+            namespaces = newReader.namespaces;
+            newReader.namespaces = null;
+
+
         }
         finally {
             newReader = null;
@@ -224,6 +235,7 @@ public class PKIStore implements VOMSTrustStore {
         signings     = new Hashtable(HASHCAPACITY);
         lscfiles     = new Hashtable(HASHCAPACITY);
         vomscerts    = new Hashtable(HASHCAPACITY);
+        namespaces   = new Hashtable(HASHCAPACITY);
 
         if (type != TYPE_VOMSDIR &&
             type != TYPE_CADIR)
@@ -250,7 +262,7 @@ public class PKIStore implements VOMSTrustStore {
         if (!theDir.exists()){
          
             if (type == TYPE_CADIR) {
-                StringBuffer message = new StringBuffer();
+                StringBuilder message = new StringBuilder();
                 message.append("Directory ");
                 message.append(dir);
                 message.append(" doesn't exist on this machine!");
@@ -290,7 +302,7 @@ public class PKIStore implements VOMSTrustStore {
         if (theDir.exists())
             load();
 
-        lastmodified = theDir.lastModified();
+
 
         if (timer) {
             theTimer = new Timer(true);
@@ -340,6 +352,7 @@ public class PKIStore implements VOMSTrustStore {
         signings     = new Hashtable(HASHCAPACITY);
         lscfiles     = new Hashtable(HASHCAPACITY);
         vomscerts    = new Hashtable(HASHCAPACITY);
+        namespaces   = new Hashtable(HASHCAPACITY);
         instances = 1;
     }
 
@@ -566,6 +579,12 @@ public class PKIStore implements VOMSTrustStore {
         signings.put(key, sp);
     }
 
+    private void load(Namespace nsp) {
+        String key = nsp.getName();
+
+	namespaces.put(key, nsp);
+    }
+
     private void load(LSCFile lsc, String vo) {
         String key = lsc.getName();
         Hashtable lscList = null;
@@ -605,6 +624,9 @@ public class PKIStore implements VOMSTrustStore {
                     else if (value == SIGN) {
                         load((SigningPolicy)c.first);
                     }
+		    else if (value == NAMESPACE) {
+		        load((Namespace)c.first);
+		    }
                 }
             }
             catch(IOException e) {
@@ -708,6 +730,10 @@ public class PKIStore implements VOMSTrustStore {
 
         }
 
+	if (f.getName().matches(".*\\.namespace")) {
+	    return new Couple(new Namespace(f), Integer.valueOf(NAMESPACE));
+	}
+
         Object o = null;
         try {
             o = PKIUtils.readObject(f);
