@@ -49,6 +49,7 @@ import java.security.cert.CertificateFactory;
 import java.security.cert.X509CRL;
 import java.security.cert.X509Certificate;
 import java.util.Arrays;
+import java.util.ArrayList;
 import java.util.List;
 import java.util.ListIterator;
 import java.util.Vector;
@@ -79,6 +80,7 @@ import org.bouncycastle.jce.X509Principal;
 import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.openssl.PEMReader;
 import org.bouncycastle.openssl.PasswordFinder;
+import org.bouncycastle.jce.provider.JCERSAPrivateKey;
 
 
 public class PKIUtils {
@@ -209,7 +211,7 @@ public class PKIUtils {
             catch(NoSuchAlgorithmException e) {
                 logger.fatal("NO MD5! " + e.getMessage(), e);
 
-                throw new IllegalStateException("NO MD5! " + e.getMessage());
+                throw new IllegalStateException("NO MD5! " + e.getMessage(), e);
             }
 
             md.update(name);
@@ -293,8 +295,7 @@ public class PKIUtils {
 
     public static String Normalize(String dn) {
         String newdn = emailPattern.matcher(dn).replaceAll("/Email");
-        newdn = uidPattern.matcher(newdn).replaceAll("/UID");
-        return newdn;
+        return uidPattern.matcher(newdn).replaceAll("/UID");
     }
 
     public static boolean DNCompare(String dn1, String dn2) {
@@ -368,7 +369,7 @@ public class PKIUtils {
 
     static private GeneralName[] getNames(GeneralNames gns) {
         DERObject obj = gns.getDERObject();
-        Vector v = new Vector();
+        ArrayList v = new ArrayList();
 
         ASN1Sequence seq = (ASN1Sequence)obj;
 
@@ -398,8 +399,6 @@ public class PKIUtils {
 
             logger.debug("Is: " + issuedIssuer.getName() +
                          " issued by " + issuerSubject.getName() + "?");
-            logger.debug("Is: " + issued.getSubjectDN().getName() +
-                         " issued by " + issuer.getSubjectDN().getName());
             logger.debug("[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[");
         }
 
@@ -407,16 +406,26 @@ public class PKIUtils {
             logger.debug("================================");
             logger.debug("issuersSubject = issuedIssuer");
 
-            //            String buggySwissSignCA="EMAILADDRESS=ca@SwissSign.com, CN=SwissSign CA (RSA IK May 6 1999 18:00:58), O=SwissSign, C=CH";
-            String buggySwissSignCA="1.2.840.113549.1.9.1=#161063614053776973735369676e2e636f6d,CN=SwissSign CA (RSA IK May 6 1999 18:00:58),O=SwissSign,C=CH";
+            boolean needtoskip = false;
+            AuthorityKeyIdentifier akid = null;
 
-            logger.debug("--"+buggySwissSignCA+"--");
-            logger.debug("--"+issuerSubject.getName()+"--"+issuerSubject.getName().getClass().getName());
-            if (!buggySwissSignCA.equals(issuerSubject.getName())) {
-                /* Skip check due to bugs in bc and SwissSign */ 
-            AuthorityKeyIdentifier akid = PKIUtils.getAKID(issued);
+            try {
+                akid = PKIUtils.getAKID(issued);
+                String s;
 
-            if (akid != null) {
+                /*  The following may throw an exception if the KeyID field 
+                    of the Authority Key Identifier is empty. */
+                if (akid != null)
+                    s = akid.toString();
+
+                needtoskip = false;
+            }
+            catch (Exception e) {
+                logger.warn("CA: " + issuerSubject.getName() + " needs to skip akid checks due to incorrectly formatted Authority Key Identifier Extension");
+                needtoskip = true;
+            }
+
+            if (!needtoskip && (akid != null)) {
                 if (logger.isDebugEnabled())
                     logger.debug("akid = " + akid);
                 
@@ -429,14 +438,14 @@ public class PKIUtils {
                     logger.debug("sid = " + skid);
 
                 if (skid != null) {
-                    logger.debug("subject Key Identifier extensions found in issuer certificate.");
+                    logger.debug("subject Key Identifier extension found in issuer certificate.");
                     logger.debug("comparing skid to akid");
 
                     byte[] skidValue = skid.getKeyIdentifier();
                     if (logger.isDebugEnabled()) {
                         logger.debug("skid");
 
-                        StringBuffer str = new StringBuffer();
+                        StringBuilder str = new StringBuilder();
                         for (int i = 0; i < skidValue.length; i++) {
                             str.append(Integer.toHexString(skidValue[i]));
                             str.append(' ');
@@ -448,7 +457,7 @@ public class PKIUtils {
                     if (logger.isDebugEnabled()) {
                         logger.debug("akid");
 
-                        StringBuffer str = new StringBuffer();
+                        StringBuilder str = new StringBuilder();
                         for (int i = 0; i < akidValue.length; i++) {
                             str.append(Integer.toHexString(akidValue[i]));
                             str.append(' ');
@@ -462,58 +471,6 @@ public class PKIUtils {
 
                     logger.debug("skid/akid check passed.");
                 }
-
-                if (false) {
-                    // The following should be skipped if the previous check passed.
-                    // And code cannot reach here unless the previous step passed.
-
-                    BigInteger sn = getAuthorityCertificateSerialNumber(akid);
-
-                    if (sn != null && !sn.equals(issuer.getSerialNumber())) {
-                        logger.error("Serial number in Authority Key Identifier and in issuer certificate do not match");
-                        logger.error("From akid              : " + sn.toString());
-                        logger.error("From issuer certificate: " + issuer.getSerialNumber());
-                        return false;
-                    }
-
-                    GeneralNames gns = getAuthorityCertIssuer(akid);
-
-                    if (gns != null) {
-                        GeneralName names[] = getNames(gns);
-
-                        int i = 0;
-
-                        while (i < names.length) {
-                            if (names[i].getTagNo() == 4) {
-                                DERObject dobj = names[i].getName().getDERObject();
-                                ByteArrayOutputStream baos = null;
-                                ASN1OutputStream aos = null;
-                                try {
-                                    baos = new ByteArrayOutputStream();
-                                    aos = new ASN1OutputStream(baos);
-                                    aos.writeObject(dobj);
-                                    aos.flush();
-                                }
-                                catch (IOException e) {
-                                    logger.error("Error in encoding of Authority Key Identifier." + e.getMessage());
-                                    return false;
-                                }
-                                X500Principal principal = new X500Principal(baos.toByteArray());
-                                X500Principal issuerIssuer  = issuer.getIssuerX500Principal();
-
-                                if (issuerIssuer.equals(principal)) {
-                                    logger.debug("PASSED");
-                                    break;
-                                }
-                                else {
-                                    logger.error("Issuer Issuer not found among Authority Key Identifier's Certifiacte Issuers.");
-                                    return false;
-                                }
-                            }
-                        }
-                    }
-                }
-            }
             }
             logger.debug("]]]]]]]]]]]]]]]]]]]]]]]]");
 
@@ -553,7 +510,7 @@ public class PKIUtils {
 
         if (logger.isDebugEnabled()) {
             if (keybytes != null) {
-                StringBuffer str = new StringBuffer();
+                StringBuilder str = new StringBuilder();
                 str.append("Real value : ");
                 for (int j =0; j < keybytes.length; j++) {
                     str.append(Integer.toHexString(keybytes[j]));
@@ -660,10 +617,10 @@ public class PKIUtils {
                     dobj = new ASN1InputStream(new ByteArrayInputStream(((DEROctetString)dobj).getOctets())).readObject();
                 }
                 catch (ClassCastException e) {
-                    throw new IllegalArgumentException("Erroneous encoding in Authority Key Identifier " + e.getMessage());
+                    throw new IllegalArgumentException("Erroneous encoding in Authority Key Identifier " + e.getMessage(), e);
                 }
                 catch (Exception e) {
-                    throw new IllegalArgumentException("While extracting Authority Key Identifier " + e.getMessage());
+                    throw new IllegalArgumentException("While extracting Authority Key Identifier " + e.getMessage(), e);
                 }
                 
                 return new AuthorityKeyIdentifier(ASN1Sequence.getInstance(dobj));
@@ -689,7 +646,7 @@ public class PKIUtils {
                     dobj = new ASN1InputStream(new ByteArrayInputStream(((DEROctetString)dobj).getOctets())).readObject();
                 }
                 catch (Exception e) {
-                    throw new IllegalArgumentException("While extracting Subject Key Identifier " + e.getMessage());
+                    throw new IllegalArgumentException("While extracting Subject Key Identifier " + e.getMessage(), e);
                 }
 
                 return SubjectKeyIdentifier.getInstance(dobj);
@@ -733,13 +690,34 @@ public class PKIUtils {
              pem = new PEMReader(new FileReader(file), finder);
              logger.debug("pem = " + pem);
              Object read = null;
+
              do {
                  read = pem.readObject();
+                 logger.debug("File is: " + file.getAbsolutePath());
                  logger.debug("Object read is: " + read);
-             } while (!(read instanceof KeyPair));
-             KeyPair pair = (KeyPair)read;
-             logger.debug("key = " + pair );
-             return pair.getPrivate();
+                 logger.debug("Object class is: " + read.getClass().getCanonicalName());
+             } while (!((read instanceof KeyPair) || (read instanceof JCERSAPrivateKey)) && read != null);
+
+             if (read != null) {
+                 if (read instanceof KeyPair) {
+                     KeyPair pair = (KeyPair)read;
+                     logger.debug("key = " + pair );
+                     return pair.getPrivate();
+                 }
+                 else if (read instanceof JCERSAPrivateKey) {
+                     logger.debug("key = " + read);
+                     return (PrivateKey)read;
+                 }
+                 else if (read instanceof PrivateKey) {
+                     return (PrivateKey)read;
+                 }
+                 else
+                     return null;
+             }
+             else {
+                 // no private key was found!
+                 return null;
+             }
          }
          catch (IOException e) {
              throw new IllegalArgumentException("Not a PEM format file " + file.getName(), e);
@@ -814,7 +792,7 @@ public class PKIUtils {
      * @throws IllegalArgumentException if the file cannot be found.
      */
     static private X509Certificate[] loadCertificates(BufferedInputStream bis) throws CertificateException, IOException {
-        List certificates = new Vector();
+        ArrayList certificates = new ArrayList();
 
         int type;
 
