@@ -58,7 +58,7 @@ import org.bouncycastle.asn1.DERObjectIdentifier;
 import org.bouncycastle.asn1.ASN1Sequence;
 import org.bouncycastle.asn1.ASN1Object;
 import org.bouncycastle.asn1.DERObject;
-import org.bouncycastle.asn1.DERInputStream;
+import org.bouncycastle.asn1.ASN1InputStream;
 import org.bouncycastle.asn1.DEROctetString;
 import org.bouncycastle.asn1.x509.X509Extension;
 import org.bouncycastle.asn1.x509.X509Extensions;
@@ -70,7 +70,7 @@ import org.glite.voms.ac.AttributeCertificateInfo;
 import org.glite.voms.ac.VOMSTrustStore;
 import org.glite.voms.contact.MyProxyCertInfo;
 
-class MyDERInputStream extends DERInputStream {
+class MyDERInputStream extends ASN1InputStream {
     public MyDERInputStream(InputStream is) {
         super(is);
     }
@@ -470,7 +470,7 @@ public class PKIVerifier {
         DERObject obj = null;
 
         try {
-            obj = new DERInputStream(new ByteArrayInputStream(ext.getValue().getOctets())).readObject();
+            obj = new ASN1InputStream(new ByteArrayInputStream(ext.getValue().getOctets())).readObject();
             MyDERInputStream str = new MyDERInputStream(((DEROctetString)obj).getOctetStream());
             int len = 0;
             int res = 0;
@@ -518,6 +518,8 @@ public class PKIVerifier {
 
         Stack certStack = new Stack();
 
+        int proxyCount = 0;
+
         // First, build the certification path
         certStack.push( certs[0] );
 
@@ -536,6 +538,11 @@ public class PKIVerifier {
 
                 logger.debug( "Is issuer" );
 
+                if (PKIUtils.isProxy(currentCert)) {
+                    logger.debug("Increase proxycount");
+                    proxyCount ++;
+                }
+
                 certStack.push( certs[i] );
                 currentCert = certs[i];
 
@@ -601,6 +608,9 @@ public class PKIVerifier {
                             certStack.push( candidate );
                             currentCert = candidate;
 
+//                             if (PKIUtils.isProxy(candidate))
+//                                 proxyCount ++;
+
                             if ( logger.isDebugEnabled() )
                                 logger.debug( "ELEMENT: "
                                         + candidate.getSubjectDN().getName() );
@@ -638,10 +648,16 @@ public class PKIVerifier {
         // now verifies it
         int stackSize = certStack.size();
         int stackPos = stackSize+1;
+        Stack constructedStack = new Stack();
+        int certCount = 0;
+
         while ( !certStack.isEmpty() ) {
             currentCert = (X509Certificate) certStack.pop();
             stackPos = stackPos -1;
 
+            if (!PKIUtils.isProxy(currentCert))
+                certCount ++;
+
             if ( logger.isDebugEnabled() )
                 logger.debug( "VERIFYING : "
                         + currentCert.getSubjectDN().getName() );
@@ -725,11 +741,11 @@ public class PKIVerifier {
 
             logger.debug( "Checking CA " + isCA );
             if ( isCA ) {
-                if ( !allowsPath( currentCert, issuerCert ) ) {
+                if ( !allowsPath( currentCert, issuerCert, constructedStack ) ) {
                     logger.error( "Certificate verification: subject '"
-                            + currentCert.getSubjectDN().getName()
-                            + "' not allowed by CA '"
-                            + issuerCert.getSubjectDN().getName() + "'" );
+                                  + currentCert.getSubjectDN().getName()
+                                  + "' not allowed by CA '"
+                                  + issuerCert.getSubjectDN().getName() + "'" );
                     return false;
                 }
 
@@ -740,7 +756,7 @@ public class PKIVerifier {
                         + " maxPath = " + maxPath );
 
                 if ( maxPath != -1 ) {
-                    if ( maxPath < certStack.size() ) {
+                    if ( maxPath < certStack.size() - proxyCount - 1  ) {
                         logger
                                 .error( "Certificate verification: Maximum certification path length exceeded." );
                         success = false;
@@ -771,6 +787,7 @@ public class PKIVerifier {
             issuerCert = currentCert;
             candidatePublicKey = currentCert.getPublicKey();
             currentLength++;
+            constructedStack.push(currentCert);
         }
 
         if ( !success )
@@ -779,7 +796,59 @@ public class PKIVerifier {
         return true;
     }
 
-    private boolean allowsPath( X509Certificate cert, X509Certificate issuer ) {
+    private boolean allowsNamespaces(X509Certificate cert, X509Certificate issuer,
+                                     Stack certStack) {
+
+        // self signed certificates always pass.
+        if (PKIUtils.selfIssued(cert))
+            return true;
+
+        Hashtable namespaces = caStore.getNamespaces();
+        Namespace namespaceCandidate = (Namespace) namespaces.get(PKIUtils.getHash(issuer));
+	  
+        if (namespaceCandidate == null) {
+            // specification says to travel up the chain in this case
+            int size = certStack.size();
+            int current = size-1;
+
+            logger.debug("size = " + size + ", current = " + current + "\n");
+            if (size > 0) {
+                do {
+                    namespaceCandidate = (Namespace)namespaces.get(PKIUtils.getHash(
+                                          (X509Certificate)certStack.elementAt(current)));
+                    current -= 1;
+                } while (namespaceCandidate == null && current != -1);
+            }
+
+            if (namespaceCandidate == null) {
+                //no candidates were found.  According to the rules, this is a permit
+                return true;
+            }
+        }
+        // at this point, namespaceCandidate != null
+
+        int index = namespaceCandidate.findIssuer(issuer);
+
+        while (index != -1) {
+            logger.debug("looking inside namespace");
+            namespaceCandidate.setCurrent(index);
+            String subject = namespaceCandidate.getSubject();
+            boolean permit = namespaceCandidate.getPermit();
+            String currentSubj = PKIUtils.getOpenSSLFormatPrincipal(issuer.getSubjectDN());
+            String currentSubjReversed = PKIUtils.getOpenSSLFormatPrincipal(issuer.getSubjectDN(), true);
+            if (subject.equals(currentSubj) || subject.equals(currentSubjReversed)) {
+                // this is the right policy
+                return permit;
+            }
+            // no decision as of yet.  Look for other rules
+            index = namespaceCandidate.findIssuer(issuer, index);
+        }
+        // candidates found, but no rule applies.  This is a deny
+        return false;
+    }
+
+    private boolean allowsPath( X509Certificate cert, X509Certificate issuer,
+				Stack certStack) {
 
         /* Self-signed certificates do not need to be checked against the signing policy. */
         if (PKIUtils.selfIssued(cert))
@@ -793,6 +862,9 @@ public class PKIVerifier {
         logger.debug("signCandidate is: " + signCandidate);
         boolean matched = false;
 
+        if (signCandidate == null)
+            return allowsNamespaces(cert, issuer, certStack);
+
         if ( signCandidate != null ) {
             logger.debug("Class of issuer is : " + issuer.getClass());
             logger.debug("Class of Subject is: " + issuer.getSubjectDN().getClass());
@@ -903,43 +975,42 @@ public class PKIVerifier {
                     } catch ( Exception e ) {
                         continue;
                     }
-                    {
-                        Set criticalExts = candidateCRL.getCriticalExtensionOIDs();
 
-                        Set permittedCriticals = new HashSet();
-                        permittedCriticals.add("2.5.29.28");
+                    Set criticalExts = candidateCRL.getCriticalExtensionOIDs();
 
-                        if ( criticalExts == null || criticalExts.isEmpty() ||
-                             (!criticalExts.isEmpty() && permittedCriticals.containsAll(criticalExts))) {
-                            if ( candidateCRL.getIssuerX500Principal().equals(
-                                    issuer.getSubjectX500Principal() ) ) {
-                                if ( candidateCRL.getNextUpdate().compareTo(
+                    Set permittedCriticals = new HashSet();
+                    permittedCriticals.add("2.5.29.28");
+
+                    if ( criticalExts == null || criticalExts.isEmpty() ||
+                         (!criticalExts.isEmpty() && permittedCriticals.containsAll(criticalExts))) {
+                        if ( candidateCRL.getIssuerX500Principal().equals(
+                                          issuer.getSubjectX500Principal() ) ) {
+                            if ( candidateCRL.getNextUpdate().compareTo(
                                         new Date() ) >= 0
-                                        && candidateCRL.getThisUpdate()
-                                                .compareTo( new Date() ) <= 0 ) {
+                                 && candidateCRL.getThisUpdate()
+                                 .compareTo( new Date() ) <= 0 ) {
 
-                                    X509CRLEntry entry = candidateCRL
-                                            .getRevokedCertificate( cert
-                                                    .getSerialNumber() );
-                                    if ( entry == null ) {
-                                        return false;
-                                    }
-                                }
-                                else {
-                                    logger.error( "CRL for CA '"+issuer.getSubjectDN().toString()+"' has expired!" );
+                                X509CRLEntry entry = candidateCRL
+                                    .getRevokedCertificate( cert
+                                        .getSerialNumber() );
+                                if ( entry == null ) {
+                                    return false;
                                 }
                             }
-                        }
-                        else {
-                            logger.error("Critical extension found in crl!");
-                            for (Iterator ii = criticalExts.iterator(); ii.hasNext();) {
-                                logger.debug("Critical CRL Extension: " +
-                                             (String)ii.next());
+                            else {
+                                logger.error( "CRL for CA '"+issuer.getSubjectDN().toString()+"' has expired!" );
                             }
-
                         }
-                        issued = true;
                     }
+                    else {
+                        logger.error("Critical extension found in crl!");
+                        for (Iterator ii = criticalExts.iterator(); ii.hasNext();) {
+                            logger.debug("Critical CRL Extension: " +
+                                         (String)ii.next());
+                        }
+
+                    }
+                    issued = true;
                 }
             }
         }
