@@ -27,7 +27,7 @@ package org.glite.voms.contact;
 
 import org.bouncycastle.asn1.DERInteger;
 import org.bouncycastle.asn1.DEREncodable;
-import org.bouncycastle.asn1.DEREncodableVector;
+import org.bouncycastle.asn1.ASN1EncodableVector;
 import org.bouncycastle.asn1.DERObject;
 import org.bouncycastle.asn1.DERObjectIdentifier;
 import org.bouncycastle.asn1.DERSequence;
@@ -35,7 +35,7 @@ import org.bouncycastle.asn1.ASN1Sequence;
 import org.bouncycastle.asn1.ASN1Object;
 import java.io.IOException;
 import java.io.ByteArrayInputStream;
-import org.bouncycastle.asn1.DERInputStream;
+import org.bouncycastle.asn1.ASN1InputStream;
 
 public class MyProxyCertInfo implements DEREncodable {
 
@@ -68,18 +68,18 @@ public class MyProxyCertInfo implements DEREncodable {
         if (seq.size() == 1) {
             // Only one element.  Must be a ProxyPolicy
             this.pathLen = -1;
-            this.policy = new ProxyPolicy((DERObjectIdentifier)(seq.getObjectAt(0)));
+            this.policy = new ProxyPolicy((ASN1Sequence)(seq.getObjectAt(0)));
         }
         else {
             // Two elements.  Which one is the first?
             DEREncodable obj = seq.getObjectAt(0);
             if (obj instanceof DERInteger) {
                 this.pathLen = ((DERInteger)obj).getValue().intValue();
-                this.policy  = new ProxyPolicy((DERObjectIdentifier)(seq.getObjectAt(0)));
+                this.policy  = new ProxyPolicy((ASN1Sequence)(seq.getObjectAt(0)));
                 this.version = VOMSProxyBuilder.GT3_PROXY;
             }
             else {
-                this.policy  = new ProxyPolicy((DERObjectIdentifier)(seq.getObjectAt(0)));
+                this.policy  = new ProxyPolicy((ASN1Sequence)(seq.getObjectAt(0)));
                 this.pathLen = ((DERInteger)obj).getValue().intValue();
                 this.version = VOMSProxyBuilder.GT4_PROXY;
             }
@@ -93,7 +93,7 @@ public class MyProxyCertInfo implements DEREncodable {
         DERObject derObj = null;
         try {
             ByteArrayInputStream inStream = new ByteArrayInputStream(payload);
-            DERInputStream derInputStream = new DERInputStream(inStream);
+            ASN1InputStream derInputStream = new ASN1InputStream(inStream);
             derObj = derInputStream.readObject();
         } catch (IOException e) {
             throw new IllegalArgumentException("Unable to convert byte array: " + 
@@ -107,7 +107,7 @@ public class MyProxyCertInfo implements DEREncodable {
     }
 
     public DERObject getDERObject() {
-        DEREncodableVector vec = new DEREncodableVector();
+        ASN1EncodableVector vec = new ASN1EncodableVector();
 
         switch(version) {
         case VOMSProxyBuilder.GT3_PROXY:
