@@ -73,6 +73,12 @@ public class VOMSESFileParser {
 
         List list = new ArrayList();
 
+        File defaultLocFile = new File( File.separator + "etc"
+                + File.separator + "vomses" );
+
+        if ( defaultLocFile.exists() )
+            list.add( defaultLocFile );
+
         if ( gliteLoc != null ) {
             File gliteLocFile = new File( gliteLoc + File.separator + "etc"
                     + File.separator + "vomses" );
@@ -108,6 +114,17 @@ public class VOMSESFileParser {
         if ( gliteVomses.exists() )
             list.add( gliteVomses );
 
+        File vomsVomses = new File( System.getProperty( "user.home" )
+                + File.separator + ".voms" + File.separator + "vomses" );
+
+	log.debug("Tried file: "+ vomsVomses.getAbsolutePath());
+	log.debug("file exists ? "+ vomsVomses.exists());
+
+        if ( vomsVomses.exists() ) {
+
+            list.add( vomsVomses );
+	}
+
         vomsesPaths = list;
 
     }
@@ -196,7 +213,7 @@ public class VOMSESFileParser {
             if ( line.matches( "\\s*$" ) )
                 continue;
 
-            String[] tokens = splitLine( line );
+            String[] tokens = splitLine( line.trim() );
 
             if ( tokens.length < 5 || tokens.length > 6 )
                 throw new VOMSException( "Syntax error on vomses file!" );
