@@ -66,7 +66,6 @@ import java.math.BigInteger;
 
 import org.apache.log4j.Logger;
 
-import org.bouncycastle.asn1.DEREncodableVector;
 import org.bouncycastle.asn1.ASN1EncodableVector;
 import org.bouncycastle.asn1.DERSequence;
 import org.bouncycastle.asn1.ASN1Sequence;
@@ -79,7 +78,7 @@ import org.bouncycastle.asn1.DERPrintableString;
 import org.bouncycastle.asn1.x509.X509Name;
 import org.bouncycastle.asn1.x509.KeyUsage;
 
-import org.bouncycastle.jce.X509V3CertificateGenerator;
+import org.bouncycastle.x509.X509V3CertificateGenerator;
 
 import org.glite.voms.ac.AttributeCertificate;
 
@@ -149,9 +148,9 @@ public class VOMSProxyBuilder {
      *
      * This methods builds an {@link AttributeCertificate} (AC) object starting from an array of bytes.
      *
-     * @param acBytes, the byte array containing the attribute certificate.
+     * @param acBytes the byte array containing the attribute certificate.
      * @return the {@link AttributeCertificate} object
-     * @throws VOMSException, in case of parsing errors.
+     * @throws VOMSException in case of parsing errors.
      */
     public static AttributeCertificate buildAC(byte[] acBytes){
 
@@ -179,12 +178,12 @@ public class VOMSProxyBuilder {
      * passed as arguments and including a list of {@link AttributeCertificate} objects that
      * will be included in the proxy.
      *
-     * @param cred, the {@link UserCredentials} from which the proxy must be created.
-     * @param ACs, the list of {@link AttributeCertificate} objects.
-     * @param lifetime, the lifetime in seconds of the generated proxy.
-     * @param version, the version of globus to which the proxy conforms
-     * @return a {@link GlobusCredential} object that represents the proxy.
-     * @throws {@link VOMSException}, if something goes wrong.
+     * @param cred the {@link UserCredentials} from which the proxy must be created.
+     * @param ACs the list of {@link AttributeCertificate} objects.
+     * @param lifetime the lifetime in seconds of the generated proxy.
+     * @param gtVersion the version of globus to which the proxy conforms
+     * @return a {@link UserCredentials} object that represents the proxy.
+     * @throws VOMSException if something goes wrong.
      *
      * @author Vincenzo Ciaschini
      * @author Andrea Ceccanti
@@ -208,7 +207,7 @@ public class VOMSProxyBuilder {
 
         Iterator i = ACs.iterator();
 
-        DEREncodableVector acVector = new DEREncodableVector();
+        ASN1EncodableVector acVector = new ASN1EncodableVector();
 
         while (i.hasNext())
             acVector.add( (AttributeCertificate)i.next() );
@@ -309,6 +308,11 @@ public class VOMSProxyBuilder {
         ProxyPolicy policy = null;
         BigInteger serialNum = null;
 
+        if (issuerKey == null) {
+            log.error("Passed issuer key is null");
+            throw new VOMSException("Passed issuerKey is null!");
+        }
+
         switch (gtVersion) {
         case GT2_PROXY:
             serialNum = cert.getSerialNumber();
@@ -379,6 +383,9 @@ public class VOMSProxyBuilder {
             }
         }
 
+        if (cnValue == null)
+            throw new IllegalArgumentException("Type of delegation unspecified");
+
         ExtensionData[] exts = (ExtensionData[])extensions.values().toArray(new ExtensionData[] {});
         for (int i = 0; i <  exts.length; i++)
             certGen.addExtension(exts[i].getOID(), exts[i].getCritical(), exts[i].getObj());
